@@ -58,7 +58,7 @@
 																							 withRunInMainThread:YES];
 	[updateTable setDisplayStatus:NO];
 	
-	NSString *description = @"Getting file list";
+	NSString *description = NSLocalizedString(@"Getting file list", nil);
 	[self addToQueue:getFiles withSubItems:[NSArray arrayWithObjects:updateTable, nil] withDescription:description];
 	[getFiles release];
 	[updateTable release];
