@@ -7,8 +7,8 @@
  */
 
 #import "DrawerController.h"
-#include "id3/globals.h"
 #import "defs.h"
+#import "TagAPI.h"
 
 // declare the private methods
 @interface DrawerController (PrivateAPI)
@@ -42,10 +42,10 @@
 - (void)awakeFromNib
 {
 	// fill genreCombo
-	NSMutableArray *genres = [[NSMutableArray alloc] init];
-	int count;
-	for (count = 0; count < ID3_NR_OF_V1_GENRES; count++)
-		[genres addObject:[NSString stringWithUTF8String:ID3_v1_genre_description[count]]];
+	
+	NSMutableDictionary* GenreDictionary = [[NSMutableDictionary alloc] initWithContentsOfFile:@"~/GenreDictionary.plist"];
+	TagAPI *tag = [[TagAPI alloc] initWithGenreList:GenreDictionary];
+	NSMutableArray *genres = [NSMutableArray arrayWithArray:[tag getGenreList]];
 	
 	[genres sortUsingSelector:@selector(compare:)];
 	
@@ -54,6 +54,8 @@
 	while (genre = [enumerator nextObject])
 		[genreCombo addItemWithObjectValue:genre];
 	
+	[tag release];
+	
 	[drawerWindow setContentSize:[preferences drawerContentSize]];
 	
 	if ([preferences showDrawer])
