@@ -543,11 +543,11 @@
 {
 	[iTunesArrayController removeAll];
 	
-	NSString *iTunesLibrary = @"~/Music/iTunes/iTunes Music Library.xml";
+	NSString *iTunesLibrary = [preferences iTunesMusicLibraryLocation];
 	iTunesLibrary = [iTunesLibrary stringByExpandingTildeInPath];
 	NSDictionary *iTunesDict = [NSDictionary dictionaryWithContentsOfFile:iTunesLibrary];
 	
-	// if we made an error loading it then give up and say we're not using iTunes
+	// if we made an error loading it then give up
 	if (iTunesDict == nil)
 	{
 		NSRunAlertPanel(NSLocalizedString(@"iTunes Library", nil), 
