@@ -13,8 +13,7 @@
 #import "NJBTransactionResult.h"
 #import "Playlist.h"
 #import "Directory.h"
-
-#include "mtp-utils.h"
+#include "libmtp.h"
 
 #define PLAYLISTS_ASSOC_NAME @"My Playlists"
 #define MUSIC_ASSOC_NAME @"Music"
@@ -39,23 +38,14 @@
 	njb_t njbs[NJB_MAX_DEVICES];
 	BOOL turbo;
 	BOOL mtpDevice;
+	LIBMTP_mtpdevice_t *device;
 	
 	NSString *deviceString;
 	NSString *firmwareVersionString;
 	NSString *deviceIDString;
 	NSString *deviceVersionString;
 	
-	PTPParams params;
-	PTP_USB ptp_usb;
-	uint8_t interfaceNumber;
-	int batteryLevelMax;
-	unsigned storageID;
-	
-	uint32_t playlistsAssoc;
-	uint32_t musicAssoc;
-	uint32_t picturesAssoc;
-	uint32_t videoAssoc;
-	uint32_t zencastAssoc;
+	uint8_t batteryLevelMax;
 }
 
 - (NJBTransactionResult *)connect;
@@ -106,8 +96,9 @@
 - (uint32_t)picturesAssoc;
 - (uint32_t)videoAssoc;
 - (uint32_t)zencastAssoc;
+- (BOOL)isAudioType:(LIBMTP_filetype_t) filetype;
 
 int progress (u_int64_t sent, u_int64_t total, const char* buf, unsigned len, void *data);
-int progressPTP(u_int32_t sent, u_int32_t total);
+int mtp_progress (uint64_t const sent, uint64_t const total, void const * const data);
 
 @end
