@@ -39,12 +39,12 @@ StatusDisplayer *statusDisplayerGlobal;
 - (NSString *)njbErrorString;
 - (NSString *)rateString:(double) rate;
 - (NSString *)uniqueFilename:(NSString *)path;
+- (LIBMTP_track_t *)libmtpTrack_tFromTrack:(Track *)track;
 // we make this private so that the user never changes the setting directly
 // so we only change it before a transfer, so it is not changed during a transfer
 // which could mess things up
 - (void)enableTurbo;
 
-- (BOOL)setMTPTrackTag:(Track *)track;
 @end
 
 @implementation NJB
@@ -56,21 +56,14 @@ StatusDisplayer *statusDisplayerGlobal;
 	if (self = [super init])
 	{
 		njb = NULL;
-		// 15 is max
+		// 15 is max (this is for libnjb devices)
 		[self setDebug:0];
 		[self setTurbo:YES];
-		// this defined for the non MTP devices
+		// this defined here for the non MTP devices
 		batteryLevelMax = 100;
-		storageID = 0;
 		
 		downloadingTracks = NO;
 		downloadingFiles = NO;
-		
-		playlistsAssoc = 0;
-		musicAssoc = 0;
-		picturesAssoc = 0;
-		videoAssoc = 0;
-		zencastAssoc = 0;
 	}
 	return self;
 }
@@ -79,6 +72,7 @@ StatusDisplayer *statusDisplayerGlobal;
 {
 	[statusDisplayerGlobal release];
 	[cachedTrackList release];
+	[cachedDataFileList release];
 	
 	[deviceString release];
 	[firmwareVersionString release];
@@ -144,27 +138,42 @@ StatusDisplayer *statusDisplayerGlobal;
 
 - (uint32_t)playlistsAssoc
 {
-	return playlistsAssoc;
+	if (device != NULL)
+		return device->default_playlist_folder;
+	else
+		return 0;
 }
 
 - (uint32_t)musicAssoc
 {
-	return musicAssoc;
+	if (device != NULL)
+		return device->default_music_folder;
+	else
+		return 0;
 }
 
 - (uint32_t)picturesAssoc
 {
-	return picturesAssoc;
+	if (device != NULL)
+		return device->default_picture_folder;
+	else
+		return 0;
 }
 
 - (uint32_t)videoAssoc
 {
-	return videoAssoc;
+	if (device != NULL)
+		return device->default_video_folder;
+	else
+		return 0;
 }
 
 - (uint32_t)zencastAssoc
 {
-	return zencastAssoc;
+	if (device != NULL)
+		return device->default_zencast_folder;
+	else
+		return 0;
 }
 
 /**********************/
@@ -245,24 +254,25 @@ StatusDisplayer *statusDisplayerGlobal;
 	{
 		mtpDevice = YES;
 	  
-		uint16_t ret = connect_first_device(&params, &ptp_usb, &interfaceNumber);
+		LIBMTP_Init();
+		device = LIBMTP_Get_First_Device();
 		
-		switch (ret)
+		if (device == NULL)
 		{
-			case PTP_CD_RC_CONNECTED:
-				connected = YES;
-				[self setStatus:STATUS_CONNECTED];
-				break;
-			case PTP_CD_RC_NO_DEVICES:
-				connected = NO;
-				errorString = NSLocalizedString(@"Could not locate any jukeboxes", nil);
-				[self setStatus:STATUS_NO_NJB];
-				break;
-			case PTP_CD_RC_ERROR_CONNECTING:
-				connected = NO;
-				errorString = NSLocalizedString(@"Could not talk to jukebox", nil);
-				[self setStatus:STATUS_COULD_NOT_OPEN];
-				break;
+			connected = NO;
+			errorString = NSLocalizedString(@"Could not locate any jukeboxes", nil);
+			[self setStatus:STATUS_NO_NJB];
+			
+			// what about couldn't connect?
+			/*
+			 errorString = NSLocalizedString(@"Could not talk to jukebox", nil);
+			 [self setStatus:STATUS_COULD_NOT_OPEN];
+			 */
+		}
+		else
+		{
+			connected = YES;
+			[self setStatus:STATUS_CONNECTED];
 		}
 	}
 	
@@ -271,21 +281,6 @@ StatusDisplayer *statusDisplayerGlobal;
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:errorString] autorelease]; 
 	}
 	
-	if (mtpDevice)
-	{
-		// get storage ID
-		PTPStorageIDs storageIDs;
-		if (ptp_getstorageids (&params, &storageIDs) == PTP_RC_OK)
-		{
-			if (storageIDs.n > 0)
-				storageID = storageIDs.Storage[0];
-			free(storageIDs.Storage);
-		}
-		
-		// check no handles array
-		params.handles.Handler = NULL;
-	}
-	
 	[self updateDiskSpace];
 	
 	// enable UTF8
@@ -295,53 +290,45 @@ StatusDisplayer *statusDisplayerGlobal;
 	// check we have reset the cache, should have been done on disconnect
 	[cachedTrackList release];
 	cachedTrackList = nil;
+	[cachedDataFileList release];
+	cachedDataFileList = nil;
 	
 	if (mtpDevice)
 	{
-		if (ptp_getdeviceinfo(&params,&(params.deviceinfo)) == PTP_RC_OK)
+		char *tempString =  LIBMTP_Get_Modelname(device);
+		deviceString = [NSString stringWithUTF8String:tempString];
+		free(tempString);
+
+		tempString = LIBMTP_Get_Serialnumber(device);
+		deviceIDString = [NSString stringWithUTF8String:tempString];
+		free(tempString);
+
+		tempString = LIBMTP_Get_Deviceversion(device);
+		NSString *versionString = [NSString stringWithUTF8String:tempString];
+		free(tempString);
+		
+		NSArray *versions = [versionString componentsSeparatedByString:@"_"];
+		if ([versions count] != 2)
 		{
-			deviceString = [NSString stringWithCString:params.deviceinfo.Model];
-			deviceIDString = [NSString stringWithCString:params.deviceinfo.SerialNumber];
-			
-			// the version string is of the form firmware_device
-			NSString *versionString = [NSString stringWithCString:params.deviceinfo.DeviceVersion];
-			NSArray *versions = [versionString componentsSeparatedByString:@"_"];
-			if ([versions count] != 2)
-			{
-				deviceVersionString = versionString;
-				firmwareVersionString = @"";
-			}
-			else
-			{
-				deviceVersionString = [versions objectAtIndex:1];
-				firmwareVersionString = [versions objectAtIndex:0];
-			}
+			deviceVersionString = versionString;
+			firmwareVersionString = @"";
 		}
 		else
 		{
-			deviceString = @"";
-			deviceVersionString = @"";
-			deviceIDString = @"";
-			firmwareVersionString = @"";
+			deviceVersionString = [versions objectAtIndex:1];
+			firmwareVersionString = [versions objectAtIndex:0];
 		}
-	
-		// get max battery level
-		PTPDevicePropDesc dpd;
 		
-		if (ptp_getdevicepropdesc(&params,PTP_DPC_BatteryLevel,&dpd) != PTP_RC_OK)
+		// unneeded
+		uint8_t currentLevel = 0;
+		if (LIBMTP_Get_Batterylevel(device, &batteryLevelMax, &currentLevel) != 0)
 		{
 			NSLog(@"Could not get battery level property description");
 			return 0;
 		}
 		
-		// if is NULL, just leave as default
-		if (dpd.FORM.Range.MaximumValue != NULL)
-			batteryLevelMax = *(uint8_t *)dpd.FORM.Range.MaximumValue;	
-		
-		ptp_free_devicepropdesc(&dpd);
-		
 		// download the data files to the cache
-		[self dataFiles];
+		//[self dataFiles];
 	}
 	else
 	{
@@ -370,10 +357,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		}
 		else
 		{
-			close_device(&ptp_usb, &params, interfaceNumber);
-			ptp_free_deviceinfo(&params.deviceinfo);
-			if (params.handles.Handler != NULL)
-				free(params.handles.Handler);
+			LIBMTP_Release_Device(device);
 		}
 		connected = NO;
 		[self setStatus:STATUS_DISCONNECTED];
@@ -381,6 +365,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		// lose the cache
 		[cachedTrackList release];
 		cachedTrackList = nil;
+		[cachedDataFileList release];
+		cachedDataFileList = nil;
 		
 		// send notification
 		[statusDisplayer postNotificationName:NOTIFICATION_DISCONNECTED object:self];
@@ -411,11 +397,11 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		unsigned short *unistring = NULL;
-		if (ptp_getdevicepropvalue(&params, PTP_DPC_DeviceFriendlyName, (void **)&unistring, PTP_DTC_UNISTR) == PTP_RC_OK)
+		char *friendlyName = LIBMTP_Get_Friendlyname(device);
+		if (friendlyName != NULL)
 		{
-			NSString *deviceFriendlyName = [UnicodeWrapper stringFromUTF16:unistring];
-			free(unistring);
+			NSString *deviceFriendlyName = [NSString stringWithUTF8String:friendlyName];
+			free(friendlyName);
 			return deviceFriendlyName;
 		}
 		else
@@ -546,128 +532,79 @@ StatusDisplayer *statusDisplayerGlobal;
 	else
 	{		
 		NSDate *date = [NSDate date];
-		PTPObjectInfo oi;
-		unsigned short *unicodevalue = NULL;
-		uint16_t *uint16value = NULL;
-		uint32_t *uint32value = NULL;
-		int ret;
-		
-		if (params.handles.Handler == NULL)
+				
+		LIBMTP_track_t *tracks = LIBMTP_Get_Tracklisting(device);
+		if (tracks == NULL)
 		{
-			if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
-			{
-				// todo: print error
-				NSLog(@"Could not get object handles...");
-				downloadingTracks = NO;
-				return nil;
-			}
+			downloadingTracks = NO;
+			return [self cachedTrackList];
 		}
-		int i = 0;
-		for (i = 0; i < params.handles.n; i++) {
-			if (ptp_getobjectinfo(&params, params.handles.Handler[i], &oi) == PTP_RC_OK)
-			{
-				if (oi.ObjectFormat == PTP_OFC_Association || 
-						(oi.ObjectFormat != PTP_OFC_WAV && oi.ObjectFormat != PTP_OFC_MP3 && oi.ObjectFormat != PTP_OFC_WMA))
-					continue;
-				
-				Track *track = [[Track alloc] init];
+		
+		LIBMTP_track_t *track, *tmp;
+		track = tracks;
+		while (track != NULL)
+		{
+			Track *myTrack = [[Track alloc] init];
+			[myTrack setItemID:track->item_id];
+			if (track->title != NULL)
+				[myTrack setTitle:[NSString stringWithUTF8String:track->title]];
+			if (track->artist != NULL)
+				[myTrack setArtist:[NSString stringWithUTF8String:track->artist]];
+			if (track->genre != NULL)
+				[myTrack setGenre:[NSString stringWithUTF8String:track->genre]];
+			if (track->album != NULL)
+				[myTrack setAlbum:[NSString stringWithUTF8String:track->album]];
 			
-				ret = ptp_getobjectpropvalue(&params, PTP_OPC_Name, params.handles.Handler[i], (void**)&unicodevalue, PTP_DTC_UNISTR);
-				if (ret == PTP_RC_OK && unicodevalue != NULL)
-				{
-					[track setTitle:[UnicodeWrapper stringFromUTF16:unicodevalue]];
-					free(unicodevalue);
-					unicodevalue = NULL;
-				}
-
-				switch (oi.ObjectFormat)
-				{
-					case PTP_OFC_WAV:
-						[track setCodec:CODEC_WAV];
-						break;
-					case PTP_OFC_MP3:
-						[track setCodec:CODEC_MP3];
-						break;
-					case PTP_OFC_WMA:
-						[track setCodec:CODEC_WMA];
-						break;
-					default:
-						[track setCodec:CODEC_UNDEFINED];
-				}
-				
-				[track setFilesize:oi.ObjectCompressedSize];
-				if (oi.Filename != NULL)
-					[track setFilename:[NSString stringWithUTF8String:oi.Filename]];
-				
-				ret = ptp_getobjectpropvalue(&params, PTP_OPC_Artist, params.handles.Handler[i], (void**)&unicodevalue, PTP_DTC_UNISTR);
-				if (ret == PTP_RC_OK && unicodevalue != NULL)
-				{
-					[track setArtist:[UnicodeWrapper stringFromUTF16:unicodevalue]];
-					free(unicodevalue);
-					unicodevalue = NULL;
-				}
-				
-				ret = ptp_getobjectpropvalue(&params, PTP_OPC_Duration, params.handles.Handler[i], (void**)&uint32value, PTP_DTC_UINT32);
-				if (ret == PTP_RC_OK && uint32value != NULL)
-				{
-					// duration is in milliseconds, we want seconds
-					[track setLength:(*uint32value)/1000];
-					free(uint32value);
-					uint32value = NULL;
-				}
-				
-				ret = ptp_getobjectpropvalue(&params, PTP_OPC_Track, params.handles.Handler[i], (void**)&uint16value, PTP_DTC_UINT16);
-				if (ret == PTP_RC_OK && uint16value != NULL)
-				{
-					[track setTrackNumber:*uint16value];
-					free(uint16value);
-					uint16value = NULL;
-				}
-				
-				ret = ptp_getobjectpropvalue(&params, PTP_OPC_Genre, params.handles.Handler[i], (void**)&unicodevalue, PTP_DTC_UNISTR);
-				if (ret == PTP_RC_OK && unicodevalue != NULL)
-				{
-					[track setGenre:[UnicodeWrapper stringFromUTF16:unicodevalue]];
-					free(unicodevalue);
-					unicodevalue = NULL;
-				}
-				
-				ret = ptp_getobjectpropvalue(&params, PTP_OPC_AlbumName, params.handles.Handler[i], (void**)&unicodevalue, PTP_DTC_UNISTR);
-				if (ret == PTP_RC_OK && unicodevalue != NULL)
-				{
-					[track setAlbum:[UnicodeWrapper stringFromUTF16:unicodevalue]];
-					free(unicodevalue);
-					unicodevalue = NULL;
-				}
-				
-				char *timeString= NULL;
-				ret = ptp_getobjectpropvalue(&params, PTP_OPC_OriginalReleaseDate, params.handles.Handler[i], (void**)&timeString, PTP_DTC_STR);
-				if (ret == PTP_RC_OK && timeString != NULL)
-				{
-					NSString *nsTimeString = [NSString stringWithCString:timeString];
-					free(timeString);
-					timeString = NULL;
-					// the Creative devices don't send seconds (but they do send tenths of a second!)
-					NSCalendarDate *date = [[NSCalendarDate alloc] initWithString:nsTimeString calendarFormat:@"%Y%m%dT%H%M"];
-					
-					[track setYear:[date yearOfCommonEra]];
-				}
-				
-				[track setItemID:params.handles.Handler[i]];
-				
-				[cachedTrackList addObject:track];
-				[track release];
-				
-				// update progress bar
-				
-				if (i % TRACKLIST_PROGRESS_UPDATE_INTERVAL == 0)
-				{
-					double progressPercent = (double)i*(double)100.0 / (double)params.handles.n;
-					
-					[statusDisplayerGlobal updateTaskProgress:progressPercent];
-				}
+			switch (track->filetype)
+			{
+				case LIBMTP_FILETYPE_WAV:
+					[myTrack setCodec:CODEC_WAV];
+					break;
+				case LIBMTP_FILETYPE_MP3:
+					[myTrack setCodec:CODEC_MP3];
+					break;
+				case LIBMTP_FILETYPE_WMA:
+					[myTrack setCodec:CODEC_WMA];
+					break;
+				default:
+					[myTrack setCodec:CODEC_UNDEFINED];
+					break;
 			}
+			
+			[myTrack setFilesize:track->filesize];
+			if (track->filename != NULL)
+				[myTrack setFilename:[NSString stringWithUTF8String:track->filename]];
+			// duration is in milliseconds
+			[myTrack setLength:track->duration/1000];
+			[myTrack setTrackNumber:track->tracknumber];
+			
+			if (track->date != NULL)
+			{
+				NSString *nsTimeString = [NSString stringWithUTF8String:track->date];
+				// the Creative devices don't send seconds (but they do send tenths of a second!)
+				NSCalendarDate *date = [[NSCalendarDate alloc] initWithString:nsTimeString calendarFormat:@"%Y%m%dT%H%M"];
+			
+				[myTrack setYear:[date yearOfCommonEra]];
+			}
+			
+			[cachedTrackList addObject:myTrack];
+			[myTrack release];
+			
+			// update progress bar
+			// TODO: libmtp does not return track count
+			/*
+			if (i % TRACKLIST_PROGRESS_UPDATE_INTERVAL == 0)
+			{
+				double progressPercent = (double)i*(double)100.0 / (double)params.handles.n;
+				
+				[statusDisplayerGlobal updateTaskProgress:progressPercent];
+			}*/
+			
+			tmp = track;
+			track = track->next;
+			LIBMTP_destroy_track_t(tmp);
 		}
+		
 		[statusDisplayerGlobal updateTaskProgress:100.0];
 		
 		double time = -[date timeIntervalSinceNow];
@@ -686,10 +623,23 @@ StatusDisplayer *statusDisplayerGlobal;
 	if (![self isConnected])
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:NSLocalizedString(@"Not connected", nil)] autorelease];
 	
+	if ([[track fullPath] length] == 0)
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:NSLocalizedString(@"Filename empty", nil)] autorelease];
+	
 	[self setStatus:STATUS_UPLOADING_TRACK];
 	
 	NSDate *date = [NSDate date];
 	
+	char* filename = NULL;
+	char* lastPathComponent = NULL;
+	
+	NS_DURING
+		filename = (char *) [[track fullPath] fileSystemRepresentation];
+		lastPathComponent = (char *) [[[track fullPath] lastPathComponent] fileSystemRepresentation];
+	NS_HANDLER
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:NSLocalizedString(@"Could not understand path %@", filename)] autorelease];
+	NS_ENDHANDLER
+	
 	if (!mtpDevice)
 	{
 		// set turbo on/off
@@ -697,7 +647,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 		unsigned int trackid;
 		njb_songid_t *songid = [self songidStructFromTrack:track];
-		if (NJB_Send_Track(njb, [[track fullPath] fileSystemRepresentation], songid, progress, NULL, &trackid) == -1 ) {
+		if (NJB_Send_Track(njb, filename, songid, progress, NULL, &trackid) == -1 ) {
 			NSString *error = [self njbErrorString];
 			NSLog(error);
 			NJB_Songid_Destroy(songid);
@@ -710,41 +660,22 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		uint32_t handle = 0;
-		uint16_t format = 0;
-			
-		switch ([track codec])
-		{
-			case CODEC_MP3:
-				format = PTP_OFC_MP3;
-				break;
-			case CODEC_WAV:
-				format = PTP_OFC_WAV;
-				break;
-			case CODEC_WMA:
-				format = PTP_OFC_WMA;
-				break;
-			default:
-				format = PTP_OFC_Undefined;
-				break;
-		}
+		LIBMTP_track_t *trackmeta = [self libmtpTrack_tFromTrack:track];
+		if (trackmeta->filename != NULL)
+			free(trackmeta->filename);
+		trackmeta->filename = strdup(filename);
 		
-		if (send_file(&params, [[track fullPath] fileSystemRepresentation], [[[track fullPath] lastPathComponent] fileSystemRepresentation],
-									format, &progressPTP, &handle, [self musicAssoc]) != PTP_RC_OK)
+		if (LIBMTP_Send_Track_From_File(device, filename, trackmeta, mtp_progress, NULL, [self musicAssoc]) != 0)
 		{
 			NSString *error = @"Error uploading file";
 			NSLog(error);
+			LIBMTP_destroy_track_t(trackmeta);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 		}
 		
-		[track setItemID:handle];
-	
-		if (![self setMTPTrackTag:track])
-		{
-			NSString *error = @"Error setting track info for file, but was uploaded correctly";
-			NSLog(error);
-			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
-		}
+		[track setItemID:trackmeta->item_id];
+		
+		 LIBMTP_destroy_track_t(trackmeta);
 	}
 	
 	double time = -[date timeIntervalSinceNow];
@@ -779,11 +710,9 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		int ret = ptp_deleteobject(&params, [track itemID],0);
-		if (ret != PTP_RC_OK)
+		if (LIBMTP_Delete_Object(device, [track itemID]) != 0)
 		{
-			ptp_perror(&params, ret);
-			NSString *error = @"error deleting file";
+			NSString *error = @"error deleting track";
 			NSLog(error);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 		}
@@ -814,10 +743,8 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		int ret = ptp_deleteobject(&params, [fileID unsignedIntValue], 0);
-		if (ret != PTP_RC_OK)
+		if (LIBMTP_Delete_Object(device, [fileID unsignedIntValue]) != 0)
 		{
-			ptp_perror(&params, ret);
 			NSString *error = @"error deleting file";
 			NSLog(error);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
@@ -837,6 +764,8 @@ StatusDisplayer *statusDisplayerGlobal;
 	
 	NSDate *date = [NSDate date];
 	
+	// todo: is fileSystemRepresentation safe here??
+	
 	if (!mtpDevice)
 	{
 		// set turbo on/off
@@ -851,8 +780,7 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		int ret = get_file(&params, [track itemID], [[track fullPath] fileSystemRepresentation], &progressPTP);
-		if (ret != 0)
+		if (LIBMTP_Get_File_To_File(device, [track itemID], [[track fullPath] fileSystemRepresentation], mtp_progress, NULL) != 0)
 		{
 			NSString *error = @"Error downloading file";
 			NSLog(error);
@@ -901,12 +829,15 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		if (![self setMTPTrackTag:newTrack])
+		LIBMTP_track_t *libmtp_track = [self libmtpTrack_tFromTrack:newTrack];
+		if (LIBMTP_Update_Track_Metadata(device, libmtp_track) != 0)
 		{
 			NSString *error = @"Could not update track info";
 			NSLog(error);
+			LIBMTP_destroy_track_t(libmtp_track);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 		}
+		LIBMTP_destroy_track_t(libmtp_track);
 	}
 	
 	[self updateDiskSpace];
@@ -920,41 +851,6 @@ StatusDisplayer *statusDisplayerGlobal;
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
-- (BOOL)setMTPTrackTag:(Track *)track
-{
-	if (!connected || !mtpDevice)
-		return NO;
-	
-	int ret = 0;
-	// todo: check all return codes
-	unsigned short *unicode = NULL;
-	unicode = [UnicodeWrapper UTF16FromString:[track title]];
-	ret = ptp_setobjectpropvalue(&params, PTP_OPC_Name, [track itemID], unicode, PTP_DTC_UNISTR);
-	free(unicode);
-	unicode = [UnicodeWrapper UTF16FromString:[track album]];
-	ret = ptp_setobjectpropvalue(&params, PTP_OPC_AlbumName, [track itemID], unicode, PTP_DTC_UNISTR);
-	free(unicode);
-	unicode = [UnicodeWrapper UTF16FromString:[track artist]];
-	ret = ptp_setobjectpropvalue(&params, PTP_OPC_Artist, [track itemID], unicode, PTP_DTC_UNISTR);
-	free(unicode);
-	unicode = [UnicodeWrapper UTF16FromString:[track genre]];
-	ret = ptp_setobjectpropvalue(&params, PTP_OPC_Genre, [track itemID], unicode, PTP_DTC_UNISTR);
-	free(unicode);
-	uint32_t durationMS = [track length] * 1000;
-	ret = ptp_setobjectpropvalue(&params, PTP_OPC_Duration, [track itemID], &durationMS, PTP_DTC_UINT32);
-	uint16_t trackNumber = [track trackNumber];
-	ret = ptp_setobjectpropvalue(&params, PTP_OPC_Track, [track itemID], &trackNumber, PTP_DTC_UINT16);
-
-	NSString *nsTimeString = [NSString stringWithFormat:@"%.04d0101T000000.0", [track year]];
-	const char *timeString = [nsTimeString cString];
-	ret = ptp_setobjectpropvalue(&params, PTP_OPC_OriginalReleaseDate, [track itemID], (void*)timeString, PTP_DTC_STR);
-	
-	if (ret != PTP_RC_OK)
-		return NO;
-	else
-		return YES;
-}
-
 - (Directory *)dataFiles
 {
 	if (![self isConnected])
@@ -1032,68 +928,37 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		PTPObjectInfo oi;
+		LIBMTP_file_t *files;
 		
-		if (params.handles.Handler == NULL)
+		files = LIBMTP_Get_Filelisting(device);
+		if (files == NULL)
 		{
-			if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
-			{
-				// todo: print error
-				NSLog(@"Could not get object handles...");
-				downloadingFiles = NO;
-				return nil;
-			}
+			downloadingFiles = NO;
+			return nil;
 		}
-		int i = 0;
-	
-		for (i = 0; i < params.handles.n; i++) {
-			if (ptp_getobjectinfo(&params,params.handles.Handler[i], &oi) == PTP_RC_OK)
+		
+		LIBMTP_file_t *file, *tmp;
+		file = files;
+		while (file != NULL)
+		{
+			// TODO: use Get_Folder_List for folders first; then get files. we will only get files here
+			
+			if (![self isAudioType:file->filetype])
 			{
-				if (oi.ObjectFormat == PTP_OFC_Association)
-				{
-					// store the base directories
-					NSString *assocName = [NSString stringWithUTF8String:oi.Filename];
-					
-					if ([assocName isEqualToString:PLAYLISTS_ASSOC_NAME])
-					{
-						playlistsAssoc = params.handles.Handler[i];
-						NSLog(@"using playlistsAssoc 0x%04x", playlistsAssoc);
-					}
-					else if ([assocName isEqualToString:MUSIC_ASSOC_NAME])
-					{
-						musicAssoc = params.handles.Handler[i];
-						NSLog(@"using musicAssoc 0x%04x", musicAssoc);
-					}
-					if ([assocName isEqualToString:PICTURES_ASSOC_NAME])
-					{
-						picturesAssoc = params.handles.Handler[i];
-						NSLog(@"using picturesAssoc 0x%04x", picturesAssoc);
-					}
-					if ([assocName isEqualToString:VIDEO_ASSOC_NAME])
-					{
-						videoAssoc = params.handles.Handler[i];
-						NSLog(@"using videoAssoc 0x%04x", videoAssoc);
-					}
-					if ([assocName isEqualToString:ZENCAST_ASSOC_NAME])
-					{
-						zencastAssoc = params.handles.Handler[i];
-						NSLog(@"using zencastAssoc 0x%04x", zencastAssoc);
-					}
-				}
-				
-				if (oi.ObjectFormat == PTP_OFC_Association ||	oi.ObjectFormat == PTP_OFC_WAV || oi.ObjectFormat == PTP_OFC_MP3 || oi.ObjectFormat == PTP_OFC_WMA)
-					continue;
-				
 				DataFile *dataFile = [[DataFile alloc] init];
-				if (oi.Filename != NULL)
-					[dataFile setFilename:[NSString stringWithUTF8String:oi.Filename]];
-				[dataFile setSize:oi.ObjectCompressedSize];
-				[dataFile setItemID:params.handles.Handler[i]];
+				if (file->filename != NULL)
+					[dataFile setFilename:[NSString stringWithUTF8String:file->filename]];
+				[dataFile setSize:file->filesize];
+				[dataFile setItemID:file->item_id];
 				
 				[baseDir addItem:dataFile toDir:nil];
 				
 				[dataFile release];
 			}
+			
+			tmp = file;
+			file = file->next;
+			LIBMTP_destroy_file_t(tmp);
 		}
 	}
 	
@@ -1124,8 +989,7 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		int ret = get_file(&params, [dataFile itemID], [[dataFile fullPath] fileSystemRepresentation], &progressPTP);
-		if (ret != 0)
+		if (LIBMTP_Get_File_To_File(device, [dataFile itemID], [[dataFile fullPath] fileSystemRepresentation], mtp_progress, NULL) != 0)
 		{
 			NSString *error = @"Error downloading file";
 			NSLog(error);
@@ -1165,7 +1029,7 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		uint16_t objectFormat = PTP_OFC_Undefined;
+		uint16_t objectFormat = LIBMTP_FILETYPE_UNKNOWN;
 		
 		uint32_t parenthandle = 0;
 		
@@ -1175,74 +1039,79 @@ StatusDisplayer *statusDisplayerGlobal;
 		NSString *extension = [[[dataFile fullPath] pathExtension] lowercaseString];
 		if ([extension isEqualToString:@"wmv"])
 		{
-			objectFormat = PTP_OFC_WMV;
+			objectFormat = LIBMTP_FILETYPE_WMV;
 			parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"avi"])
 		{
-			objectFormat = PTP_OFC_AVI;
+			objectFormat = LIBMTP_FILETYPE_AVI;
 			parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"mpg"] || [extension isEqualToString:@"mpeg"])
 		{
-			objectFormat = PTP_OFC_MPEG;
+			objectFormat = LIBMTP_FILETYPE_MPEG;
 			parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"asf"])
 		{
-			objectFormat = PTP_OFC_ASF;
+			objectFormat = LIBMTP_FILETYPE_ASF;
 			parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"qt"])
 		{
-			objectFormat = PTP_OFC_QT;
+			objectFormat = LIBMTP_FILETYPE_QT;
 			parenthandle = [self videoAssoc];
 		}
 		else if ([extension isEqualToString:@"jpg"] || [extension isEqualToString:@"jpeg"])
 		{
-			objectFormat = PTP_OFC_EXIF_JPEG;
+			objectFormat = LIBMTP_FILETYPE_JPEG;
 			parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"tif"] || [extension isEqualToString:@"tiff"])
 		{
-			objectFormat = PTP_OFC_TIFF;
+			objectFormat = LIBMTP_FILETYPE_TIFF;
 			parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"bmp"])
 		{
-			objectFormat = PTP_OFC_BMP;
+			objectFormat = LIBMTP_FILETYPE_BMP;
 			parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"gif"])
 		{
-			objectFormat = PTP_OFC_GIF;
+			objectFormat = LIBMTP_FILETYPE_GIF;
 			parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"pict"])
 		{
-			objectFormat = PTP_OFC_PICT;
+			objectFormat = LIBMTP_FILETYPE_PICT;
 			parenthandle = [self picturesAssoc];
 		}
 		else if ([extension isEqualToString:@"png"])
 		{
-			objectFormat = PTP_OFC_PNG;
+			objectFormat = LIBMTP_FILETYPE_PNG;
 			parenthandle = [self picturesAssoc];
 		}
 		
 		NSLog(@"uploading file with object format 0x%.02x", objectFormat);
 		
-		uint32_t handle = 0;
-		uint16_t ret = send_file(&params, [[dataFile fullPath] fileSystemRepresentation], [[[dataFile fullPath] lastPathComponent] fileSystemRepresentation],
-														 objectFormat, &progressPTP, &handle, parenthandle);
-		if (ret != PTP_RC_OK)
+		LIBMTP_file_t *genfile = LIBMTP_new_file_t();
+		genfile->filesize = [dataFile size];
+		genfile->filetype = objectFormat;
+		// todo: safe?
+		genfile->filename = strdup([[[dataFile fullPath] lastPathComponent] fileSystemRepresentation]);
+		
+		if (LIBMTP_Send_File_From_File(device, [[dataFile fullPath] fileSystemRepresentation], genfile, mtp_progress, NULL, parenthandle) != 0)
 		{
 			/* we might get an (MTP error) object too large here */
 			NSString *error = @"Error uploading file";
 			NSLog(error);
+			LIBMTP_destroy_file_t(genfile);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 		}
-		
-		[dataFile setItemID:handle];
+
+		[dataFile setItemID:genfile->item_id];
+		LIBMTP_destroy_file_t(genfile);
 	}
 	
 	[self updateDiskSpace];
@@ -1273,16 +1142,12 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		PTPStorageInfo storageInfo;
-		if (ptp_getstorageinfo(&params, storageID, &storageInfo) == PTP_RC_OK)
+		char *storage_description = NULL;
+		char *volume_label = NULL;
+		if (LIBMTP_Get_Storageinfo(device, &totalDiskSpace, &freeDiskSpace, &storage_description, &volume_label) == 0)
 		{
-			freeDiskSpace = storageInfo.FreeSpaceInBytes;
-			totalDiskSpace = storageInfo.MaxCapability;
-			
-			if (storageInfo.StorageDescription != NULL)
-				free(storageInfo.StorageDescription);
-			if (storageInfo.VolumeLabel != NULL)
-				free(storageInfo.VolumeLabel);
+			free(storage_description);
+			free(volume_label);
 			
 			[statusDisplayer updateDiskSpace:totalDiskSpace withFreeSpace:freeDiskSpace];
 		}
@@ -1304,8 +1169,18 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		// note Creative devices seem not to support this
+		/*char *secureTime = NULL;
+		if (LIBMTP_Get_Secure_Time(device, &secureTime) != 0)
+			return nil;
 		
+		if (secureTime == NULL)
+			return nil;
+		
+		NSLog(@"Secure time: %s", secureTime);
+		
+		free(secureTime);*/
+		// TODO
+		/*
 		if (!ptp_property_issupported(&params, PTP_DPC_DateTime))
 			return nil;
 		
@@ -1321,7 +1196,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 			return [date autorelease];
 		}
-		else
+		else*/
 			return nil;
 	}
 }
@@ -1347,12 +1222,9 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		unsigned short *unistring = [UnicodeWrapper UTF16FromString:owner];
-		uint16_t ret = ptp_setdevicepropvalue(&params, PTP_DPC_DeviceFriendlyName, unistring, PTP_DTC_UNISTR);
-		free(unistring);
-		if (ret != PTP_RC_OK)
+		if (LIBMTP_Set_Friendlyname(device, [owner UTF8String]) != 0)
 		{
-			NSString *error = [NSString stringWithFormat:@"Could not set device friendly name, return value 0x%04.x", ret];
+			NSString *error = [NSString stringWithFormat:@"Could not set device friendly name"];
 			NSLog(error);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 		}
@@ -1441,65 +1313,37 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		PTPObjectInfo oi;
-		
-		// we can't get by type because unsupported on Creative devices (we only want PTP_OFC_AbstractAudioVideoPlaylist)
-		if (params.handles.Handler == NULL)
+		LIBMTP_playlist_t *libmtpPlaylists = LIBMTP_Get_Playlist_List(device);
+		if (libmtpPlaylists == NULL)
 		{
-			if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
-			{
-				// todo: print error
-				NSLog(@"Could not get playlist object handles...");
-				return nil;
-			}
+			// no playlists
+			return [playlists autorelease];
 		}
-		int i = 0;
-		for (i = 0; i < params.handles.n; i++) {
-			if (ptp_getobjectinfo(&params,params.handles.Handler[i], &oi) == PTP_RC_OK)
+		LIBMTP_playlist_t *pl, *tmp;
+    pl = libmtpPlaylists;
+    while (pl != NULL) {
+			Playlist *newPlaylist = [[Playlist alloc] init];
+			
+			[newPlaylist setPlaylistID:pl->playlist_id];
+			[newPlaylist setName:[NSString stringWithUTF8String:pl->name]];
+			int i = 0;
+			for (i = 0; i < pl->no_tracks; i++)
 			{
-				if (oi.ObjectFormat != PTP_OFC_AbstractAudioVideoPlaylist)
-					continue;
-				
-				Playlist *newPlaylist = [[Playlist alloc] init];
-				
-				unsigned short *unicodevalue = NULL;
-				uint16_t ret = 0;
-				ret = ptp_getobjectpropvalue(&params, PTP_OPC_Name, params.handles.Handler[i], (void**)&unicodevalue, PTP_DTC_UNISTR);
-				if (ret == PTP_RC_OK && unicodevalue != NULL)
-				{
-					[newPlaylist setName:[UnicodeWrapper stringFromUTF16:unicodevalue]];
-					free(unicodevalue);
-					unicodevalue = NULL;
-				}
-				
-				uint32_t* ohArray = NULL;
-				uint32_t arrayLen = 0;
-				if (ptp_getobjectreferences (&params, params.handles.Handler[i], &ohArray, &arrayLen) == PTP_RC_OK)
-				{
-					int j = 0;
-					for (j = 0; j<arrayLen; j++)
-					{
-						Track *newTrack = [[Track alloc] init];
-						[newTrack setItemID:ohArray[j]];
-						[newPlaylist addTrack:newTrack];
-						[newTrack release];
-					}
-					free(ohArray);
-				}
-				else
-				{
-					NSLog(@"Could not get playlist tracks for playlist handle 0x%08.x", params.handles.Handler[i]);
-				}
-				
-				[newPlaylist setPlaylistID:params.handles.Handler[i]];
-				
-				// todo: worry about playlist statuses
-				[newPlaylist setState:NJB_PL_UNCHANGED];
-				
-				[playlists addObject:newPlaylist];
-				[newPlaylist release];
+				Track *newTrack = [[Track alloc] init];
+				[newTrack setItemID:pl->tracks[i]];
+				[newPlaylist addTrack:newTrack];
+				[newTrack release];
 			}
-		}
+			
+			[newPlaylist setState:NJB_PL_UNCHANGED];
+			
+			[playlists addObject:newPlaylist];
+			[newPlaylist release];
+
+      tmp = pl;
+      pl = pl->next;
+      LIBMTP_destroy_playlist_t(tmp);
+    }
 	}
 	return [playlists autorelease];
 }
@@ -1555,11 +1399,14 @@ StatusDisplayer *statusDisplayerGlobal;
 		uint32_t size = [tracks count];
 		uint32_t *trackArray = (uint32_t*)malloc(sizeof(uint32_t)*size);
 		uint32_t i = 0;
-		uint16_t ret = 0;
 		for (i = 0; i < size; i++)
 			trackArray[i] = [[tracks objectAtIndex:i] itemID];
-		// store the ID so we can unlock
-		uint32_t playlistID = [playlist playlistID];
+		
+		LIBMTP_playlist_t *libmtpPlaylist = LIBMTP_new_playlist_t();
+		libmtpPlaylist->name = strdup([[playlist name] UTF8String]);
+		libmtpPlaylist->playlist_id = [playlist playlistID];
+		libmtpPlaylist->no_tracks = size;
+		libmtpPlaylist->tracks = trackArray;
 		
 		int playlistState = [playlist state];
 		[playlist setState:NJB_PL_UNCHANGED];
@@ -1567,68 +1414,31 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 		if (playlistState == NJB_PL_NEW)
 		{
-			/* note that we really want to create a new playlist not by using
-			SendObjectInfo but with the enhanced SendObjectPropList. Then we can send
-			filesize 0 (we still have to send object, but with no data). Sending filesize
-			0 with SendObjectInfo gives error PTP_RC_StoreFull. So hopefully we can persuade
-			Microsoft to let us use SendObjectPropList. */
-			
-			PTPObjectInfo newInfo;
-			memset(&newInfo, 0, sizeof(newInfo));
-			[playlist lock];
-			newInfo.Filename = [[NSString stringWithFormat:@"%@.zpl", [playlist name]] cString];
-			[playlist unlock];
-			newInfo.ObjectFormat = PTP_OFC_AbstractAudioVideoPlaylist;
-			newInfo.ObjectCompressedSize = 1;
-			
-			uint32_t store = 0;
-			uint32_t parenthandle = 0; // don't set to PLAYLISTS_ASSOC yet since we have to check it exists first as sendobjectinfo fails if doesn't
-			
-			ret = ptp_sendobjectinfo(&params, &store, &parenthandle, &playlistID, &newInfo);
-			if (ret != PTP_RC_OK)
+			if (LIBMTP_Create_New_Playlist(device, libmtpPlaylist, device->default_playlist_folder) != 0)
 			{
-				free(trackArray);
+				LIBMTP_destroy_playlist_t(libmtpPlaylist);
 				return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:
-					[NSString stringWithFormat:@"Could not create new playlist, error 0x%.04x", ret]] autorelease];
+					[NSString stringWithFormat:@"Could not create new playlist"]] autorelease];
 			}
-			char *data='\0';
-			ret = ptp_sendobject(&params, data, 1);
-			if (ret != PTP_RC_OK)
-			{
-				free(trackArray);
-				return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:
-					[NSString stringWithFormat:@"Could not create new playlist data, error 0x%.04x", ret]] autorelease];
-			}
-			
 			[playlist lock];
-			[playlist setPlaylistID:playlistID];
+			[playlist setPlaylistID:libmtpPlaylist->playlist_id];
 			[playlist unlock];
 		}
-		
-		if (playlistState == NJB_PL_NEW || playlistState == NJB_PL_CHNAME)
+		else
 		{
-			unsigned short *unicode = NULL;
-			[playlist lock];
-			unicode = [UnicodeWrapper UTF16FromString:[playlist name]];
-			[playlist unlock];
-			ret = ptp_setobjectpropvalue(&params, PTP_OPC_Name, playlistID, unicode, PTP_DTC_UNISTR);
-			free(unicode);
-			if (ret != PTP_RC_OK)
+			if (LIBMTP_Update_Playlist(device, libmtpPlaylist) != 0)
 			{
-				free(trackArray);
+				LIBMTP_destroy_playlist_t(libmtpPlaylist);
 				return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:
-					[NSString stringWithFormat:@"Could not set name for new playlist, error 0x%.04x", ret]] autorelease];
+					[NSString stringWithFormat:@"Could not update playlist"]] autorelease];
 			}
+			[playlist lock];
+			// it doesn't change the playlist now but put it in to be
+			// on the safe side
+			[playlist setPlaylistID:libmtpPlaylist->playlist_id];
+			[playlist unlock];
 		}
-		
-		ret = ptp_setobjectreferences(&params, playlistID, trackArray, size);
-		if (ret != PTP_RC_OK)
-		{
-			free(trackArray);
-			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:
-				[NSString stringWithFormat:@"Could not update playlist, error 0x%.04x", ret]] autorelease];
-		}
-		free(trackArray);
+		LIBMTP_destroy_playlist_t(libmtpPlaylist);
 	}
 	
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
@@ -1651,8 +1461,7 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		int ret = ptp_deleteobject(&params, [playlist playlistID],0);
-		if (ret != PTP_RC_OK)
+		if (LIBMTP_Delete_Object(device, [playlist playlistID]) != 0)
 		{
 			NSString *error = @"error deleting playlist";
 			NSLog(error);
@@ -1674,10 +1483,7 @@ int progress(u_int64_t sent, u_int64_t total, const char* buf, unsigned len, voi
 	return 0;
 }
 
-/* called by the file/track transfer functions of libptp
- * to tell us the progress
- */
-int progressPTP(u_int32_t sent, u_int32_t total)
+int mtp_progress (uint64_t const sent, uint64_t const total, void const * const data)
 {
 	double percent = (double)sent*100.0/(double)total;
 	[statusDisplayerGlobal updateTaskProgress:percent];
@@ -1872,21 +1678,14 @@ int progressPTP(u_int32_t sent, u_int32_t total)
 	}
 	else
 	{				
-		uint8_t *value = NULL;
-		if (ptp_getdevicepropvalue(&params,PTP_DPC_BatteryLevel,(void**)&value,PTP_DTC_UINT8) != PTP_RC_OK)
+		uint8_t value = 0;
+		if (LIBMTP_Get_Batterylevel(device, &batteryLevelMax, &value) != 0)
 		{
 			NSLog(@"Could not get battery level property value");
 			return 0;
 		}
 				
-		if (value != NULL)
-		{
-			int level = *value;
-			free(value);
-			return level;
-		}
-		else
-			return 0;
+		return value;
 	}
 }
 
@@ -1906,6 +1705,7 @@ int progressPTP(u_int32_t sent, u_int32_t total)
 	else
 	{
 		// they always charge...
+		// TODO: they don't - they can be fully charged. Can we see that?
 		return @"Charging";
 	}
 }
@@ -1920,4 +1720,56 @@ int progressPTP(u_int32_t sent, u_int32_t total)
 		NJB_Set_Turbo_Mode(njb, NJB_TURBO_OFF);
 }
 
+// TODO: libmtp has this in LIBMTP_Get_Tracklisting - they should be the same function
+- (BOOL)isAudioType:(LIBMTP_filetype_t) filetype
+{
+	if (filetype == LIBMTP_FILETYPE_WAV ||
+			filetype == LIBMTP_FILETYPE_MP3 ||
+			filetype == LIBMTP_FILETYPE_WMA ||
+			filetype == LIBMTP_FILETYPE_OGG ||
+			filetype == LIBMTP_FILETYPE_AUDIBLE ||
+			filetype == LIBMTP_FILETYPE_MP4 ||
+			filetype == LIBMTP_FILETYPE_UNDEF_AUDIO
+			)
+		return YES;
+	else
+		return NO;
+}
+
+// return value must be freed (LIBMTP_destroy_track_t) by caller
+- (LIBMTP_track_t *)libmtpTrack_tFromTrack:(Track *)track
+{
+	LIBMTP_track_t *libmtp_track = LIBMTP_new_track_t();
+	
+	switch ([track codec])
+	{
+		case CODEC_MP3:
+			libmtp_track->filetype = LIBMTP_FILETYPE_MP3;
+			break;
+		case CODEC_WAV:
+			libmtp_track->filetype = LIBMTP_FILETYPE_WAV;
+			break;
+		case CODEC_WMA:
+			libmtp_track->filetype = LIBMTP_FILETYPE_WMA;
+			break;
+		default:
+			libmtp_track->filetype = LIBMTP_FILETYPE_UNDEF_AUDIO;
+			break;
+	}
+		
+	libmtp_track->title = strdup([[track title] UTF8String]);
+	libmtp_track->album = strdup([[track album] UTF8String]);
+	libmtp_track->artist = strdup([[track artist] UTF8String]);
+	libmtp_track->genre = strdup([[track genre] UTF8String]);
+	NSString *nsTimeString = [NSString stringWithFormat:@"%.04d0101T000000.0", [track year]];
+	libmtp_track->date = strdup([nsTimeString UTF8String]);
+	libmtp_track->tracknumber = [track trackNumber];
+	libmtp_track->duration = [track length] * 1000;
+	libmtp_track->filesize = [track filesize];
+	libmtp_track->filename = strdup([[track filename] UTF8String]);
+	libmtp_track->item_id = [track itemID];
+		
+	return libmtp_track;
+}
+
 @end
